package com.aliyun.vodplayer.demo.activity;

import android.content.Context;
import android.os.Build;
import android.support.annotation.Nullable;
import android.support.annotation.RequiresApi;
import android.text.TextUtils;
import android.util.AttributeSet;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.Toast;

import com.aliyun.vodplayer.demo.R;
import com.aliyun.vodplayer.demo.utils.PlayAuthUtil;
import com.aliyun.vodplayer.demo.utils.WrapCheckGroup;
import com.aliyun.vodplayer.downloader.AliyunDownloadInfoListener;
import com.aliyun.vodplayer.downloader.AliyunDownloadManager;
import com.aliyun.vodplayer.downloader.AliyunDownloadMediaInfo;
import com.aliyun.vodplayer.media.AliyunDataSource;
import com.aliyun.vodplayer.media.AliyunPlayAuth;
import com.aliyun.vodplayer.media.IAliyunVodPlayer;

import org.json.JSONObject;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.lang.ref.WeakReference;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by lifujun on 2017/6/22.
 */

public class AddDownloadView extends LinearLayout {

    private EditText vid, auth;
    private Button prepareBtn, downloadBtn;
    private WrapCheckGroup wrapCheckGroup;

    private String mVid = null;
    private String mAuthinfo = null;

    private AliyunDownloadManager downloadManager;

    private Map<String, String> qualityList = new HashMap<>();

    public AddDownloadView(Context context) {
        super(context);
        init();
    }


    public AddDownloadView(Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
        init();
    }


    public AddDownloadView(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init();
    }

    AliyunDownloadInfoListener downloadInfoListener ;

    private void init() {
        LayoutInflater.from(getContext()).inflate(R.layout.view_add_download, this, true);
        findAllViews();
        downloadManager = AliyunDownloadManager.getInstance(getContext());
        downloadInfoListener = new MyDownloadInfoListener(this) {

            @Override
            public void onPrepared(List<AliyunDownloadMediaInfo> list) {
                //第二步 ； 准备结束
                showAllDownloadItems(list);
            }
        };
        downloadManager.addDownloadInfoListener(downloadInfoListener);

        qualityList.put(IAliyunVodPlayer.QualityValue.QUALITY_FLUENT, "流畅");
        qualityList.put(IAliyunVodPlayer.QualityValue.QUALITY_LOW, "标清");
        qualityList.put(IAliyunVodPlayer.QualityValue.QUALITY_STAND, "高清");
        qualityList.put(IAliyunVodPlayer.QualityValue.QUALITY_HIGH, "超清");
        qualityList.put(IAliyunVodPlayer.QualityValue.QUALITY_2K, "2k");
        qualityList.put(IAliyunVodPlayer.QualityValue.QUALITY_4K, "4k");
        qualityList.put(IAliyunVodPlayer.QualityValue.QUALITY_ORIGINAL, "原画");
    }

    public void destory() {
        downloadManager.removeDownloadInfoListener(downloadInfoListener);
        downloadInfoListener = null;
    }

    private class MyDownloadInfoListener implements AliyunDownloadInfoListener {

        private WeakReference<AddDownloadView> weakView;

        public MyDownloadInfoListener(AddDownloadView view) {
            weakView = new WeakReference<AddDownloadView>(view);
        }

        @Override
        public void onPrepared(List<AliyunDownloadMediaInfo> infos) {
            AddDownloadView view = weakView.get();
            if (view != null) {
                view.downloadInfoListener.onPrepared(infos);
            }
        }

        @Override
        public void onStart(AliyunDownloadMediaInfo info) {

        }

        @Override
        public void onProgress(AliyunDownloadMediaInfo info, int percent) {

        }

        @Override
        public void onStop(AliyunDownloadMediaInfo info) {

        }

        @Override
        public void onCompletion(AliyunDownloadMediaInfo info) {

        }

        @Override
        public void onError(AliyunDownloadMediaInfo info, int code, String msg) {

        }

        @Override
        public void onWait(AliyunDownloadMediaInfo outMediaInfo) {

        }

        @Override
        public void onM3u8IndexUpdate(AliyunDownloadMediaInfo outMediaInfo, int index) {

        }
    }

    private void showAllDownloadItems(List<AliyunDownloadMediaInfo> list) {
        if (list == null) {
            Toast.makeText(getContext(), "无权限下载", Toast.LENGTH_LONG).show();
            return;
        }

        Log.d("demo", "list size = " + list.size());

        for (AliyunDownloadMediaInfo info : list) {
            String encript = info.isEncripted() == 1 ? " 加密" : " 未加密";
            CheckBox item = (CheckBox) LayoutInflater.from(getContext()).inflate(R.layout.view_item_quality, null, false);
            item.setText(" " + qualityList.get(info.getQuality()) + " {" + info.getFormat() + "," + formatSize(info.getSize()) + encript + "} ");
            item.setTag(info);
            wrapCheckGroup.addCheckBox(item);
        }
    }

    private String formatSize(int size) {
        int kb = (int) (size / 1024f);
        if (kb < 1024) {
            return kb + "KB";
        }

        int mb = (int) (kb / 1024f);
        return mb + "MB";

    }

    private void findAllViews() {
        vid = (EditText) findViewById(R.id.vid);
        auth = (EditText) findViewById(R.id.auth);


        prepareBtn = (Button) findViewById(R.id.prepare);
        downloadBtn = (Button) findViewById(R.id.download);

        wrapCheckGroup = (WrapCheckGroup) findViewById(R.id.quality_list);
        wrapCheckGroup.removeCheckBox();

        prepareBtn.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                wrapCheckGroup.removeCheckBox();
                prepareDownload();
            }
        });

        downloadBtn.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                downloadItem();
            }
        });

    }

    private void downloadItem() {
        CheckBox checkedItem = wrapCheckGroup.getSelectedBox();
        if (checkedItem == null) {
            Toast.makeText(getContext(), "请选择一个清晰度进行下载", Toast.LENGTH_LONG).show();
            return;
        }
        AliyunDownloadMediaInfo info = (AliyunDownloadMediaInfo) checkedItem.getTag();
        AliyunPlayAuth.AliyunPlayAuthBuilder adb = getAliyunPlayAuthBuilder();
        adb.setFormat(info.getFormat());
        adb.setQuality(info.getQuality());
        adb.setIsEncripted(info.isEncripted());
        if (adb == null) return;
        if (onViewClickListener != null) {
            onViewClickListener.onDownload(adb.build(), info);
        }

    }

    //第一步；准备：
    private void prepareDownload() {
        AliyunPlayAuth.AliyunPlayAuthBuilder adb = getAliyunPlayAuthBuilder();
        if (adb == null) return;
        downloadManager.prepareDownloadMedia(adb.build());
    }

    @Nullable
    private AliyunPlayAuth.AliyunPlayAuthBuilder getAliyunPlayAuthBuilder() {
        mVid = vid.getText().toString();
        mAuthinfo = auth.getText().toString();
        if (TextUtils.isEmpty(mVid)) {
            Toast.makeText(getContext(), "有信息没有填写", Toast.LENGTH_LONG).show();
            return null;
        }

        if (TextUtils.isEmpty(mVid) ||
                TextUtils.isEmpty(mAuthinfo)) {
            Toast.makeText(getContext(), "有信息没有填写", Toast.LENGTH_LONG).show();
            return null;
        }

        AliyunPlayAuth.AliyunPlayAuthBuilder adb = new AliyunPlayAuth.AliyunPlayAuthBuilder();
        adb.setVid(mVid);
        adb.setPlayAuth(mAuthinfo);
        return adb;
    }

    private OnViewClickListener onViewClickListener;

    public void setOnViewClickListener(OnViewClickListener l) {
        onViewClickListener = l;
    }

    public interface OnViewClickListener {
        void onCancel();

        void onDownload(AliyunPlayAuth build, AliyunDownloadMediaInfo info);
    }

}
