package com.aliyun.vodplayer.demo.activity;

import android.app.Dialog;
import android.content.DialogInterface;
import android.os.Build;
import android.os.Bundle;
import android.support.annotation.RequiresApi;
import android.support.v7.app.AppCompatActivity;
import android.view.Display;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.ListView;
import android.widget.Toast;

import com.aliyun.vodplayer.demo.R;
import com.aliyun.vodplayer.downloader.AliyunDownloadInfoListener;
import com.aliyun.vodplayer.downloader.AliyunDownloadManager;
import com.aliyun.vodplayer.downloader.AliyunDownloadMediaInfo;
import com.aliyun.vodplayer.media.AliyunPlayAuth;

import java.util.List;

public class DemoDownloadActivity extends AppCompatActivity {

    private static final String TAG = DemoDownloadActivity.class.getSimpleName();
    private ListView downloadListView;
    private DownloadAdapter downloadAdapter;
    private AliyunDownloadManager downloadManager;

//    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_demo_download);

        findAllViews();
        downloadManager = AliyunDownloadManager.getInstance(this);

        downloadManager.addDownloadInfoListener(downloadInfoListener);

        //如果是要恢复的话，需要注意playAuth的时效性
//      List<AliyunDownloadMediaInfo> downloadingMedias = downloadManager.getDownloadingMedias();
//        downloadAdapter.addAllInfos(downloadingMedias);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        //移除防止内存泄漏
        downloadManager.removeDownloadInfoListener(downloadInfoListener);
    }

    private void findAllViews() {
        downloadListView = (ListView) findViewById(R.id.download_list);
        downloadAdapter = new DownloadAdapter(this);
        downloadListView.setAdapter(downloadAdapter);

        findViewById(R.id.add_download).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                showAddDownloadView();
            }
        });
    }

    private void showAddDownloadView() {
        Dialog dialog = new Dialog(this/*,R.style.addDownloadDialog*/);
        final AddDownloadView contentView = new AddDownloadView(this);
        dialog.setTitle("选择下载项");
        contentView.setOnViewClickListener(viewClickListener);
        dialog.setContentView(contentView);
/*
         * 将对话框的大小按屏幕大小的百分比设置
         */
        WindowManager m = getWindowManager();
        Display d = m.getDefaultDisplay(); // 获取屏幕宽、高用
        Window dialogWindow = dialog.getWindow();
        WindowManager.LayoutParams p = dialogWindow.getAttributes(); // 获取对话框当前的参数值
        p.width = (int) (d.getWidth() * 0.96); // 宽度设置
        dialogWindow.setAttributes(p);
        dialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
            @Override
            public void onDismiss(DialogInterface dialogInterface) {
                contentView.destory();
            }
        });
        dialog.show();
    }

    private Dialog dialog = null;

    private AddDownloadView.OnViewClickListener viewClickListener = new AddDownloadView.OnViewClickListener() {
        @Override
        public void onCancel() {
            if (dialog != null) {
                dialog.dismiss();
            }
        }

        @Override
        public void onDownload(AliyunPlayAuth build, AliyunDownloadMediaInfo info) {
            if (dialog != null) {
                dialog.dismiss();
            }
            addNewInfo(info, build);
        }
    };

    private void addNewInfo(AliyunDownloadMediaInfo info, AliyunPlayAuth build) {
            downloadAdapter.addNewInfo(info, build);
    }

    private AliyunDownloadInfoListener downloadInfoListener = new AliyunDownloadInfoListener() {


        @Override
        public void onPrepared(List<AliyunDownloadMediaInfo> list) {

        }

        @Override
        public void onStart(AliyunDownloadMediaInfo aliyunDownloadMediaInfo) {
            downloadAdapter.updateInfo(aliyunDownloadMediaInfo);
            downloadAdapter.notifyDataSetChanged();
        }

        @Override
        public void onProgress(AliyunDownloadMediaInfo aliyunDownloadMediaInfo, int i) {

            downloadAdapter.updateInfo(aliyunDownloadMediaInfo);
            downloadAdapter.notifyDataSetChanged();
        }

        @Override
        public void onStop(AliyunDownloadMediaInfo aliyunDownloadMediaInfo) {

            downloadAdapter.updateInfo(aliyunDownloadMediaInfo);
            downloadAdapter.notifyDataSetChanged();
        }

        @Override
        public void onCompletion(AliyunDownloadMediaInfo aliyunDownloadMediaInfo) {

            downloadAdapter.updateInfo(aliyunDownloadMediaInfo);
            downloadAdapter.notifyDataSetChanged();
        }

        @Override
        public void onError(AliyunDownloadMediaInfo aliyunDownloadMediaInfo, int i, String s) {

            downloadAdapter.updateInfo(aliyunDownloadMediaInfo);
            downloadAdapter.notifyDataSetChanged();
            Toast.makeText(DemoDownloadActivity.this.getApplicationContext(), s , Toast.LENGTH_LONG).show();
        }

        @Override
        public void onWait(AliyunDownloadMediaInfo aliyunDownloadMediaInfo) {
            downloadAdapter.updateInfo(aliyunDownloadMediaInfo);
            downloadAdapter.notifyDataSetChanged();
        }

        @Override
        public void onM3u8IndexUpdate(AliyunDownloadMediaInfo outMediaInfo, int index) {

        }
    };




}
