package com.aliyun.vodplayer.demo.activity;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.AsyncTask;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import com.aliyun.vodplayer.demo.R;
import com.aliyun.vodplayer.demo.utils.PlayAuthUtil;
import com.aliyun.vodplayer.downloader.AliyunDownloadManager;
import com.aliyun.vodplayer.downloader.AliyunDownloadMediaInfo;
import com.aliyun.vodplayer.downloader.AliyunRefreshPlayAuthCallback;
import com.aliyun.vodplayer.media.AliyunErrorCode;
import com.aliyun.vodplayer.media.AliyunPlayAuth;
import com.aliyun.vodplayer.utils.VcPlayerLog;
import com.bumptech.glide.Glide;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * Created by lifujun on 2017/9/14.
 */

public class DownloadAdapter extends BaseAdapter {

    private static final String TAG = DownloadAdapter.class.getSimpleName();
    private final AliyunDownloadManager downloadManager;
    private Context mContext;

    public DownloadAdapter(Context context) {
        mContext = context.getApplicationContext();

        downloadManager = AliyunDownloadManager.getInstance(mContext.getApplicationContext());
    }

    public void addNewInfo(AliyunDownloadMediaInfo info, AliyunPlayAuth build) {
        if (!hasAdded(info)) {
            downloadSourceHashMap.put(info, build);
            downloadItems.add(info);

            notifyDataSetChanged();
        }
    }

////    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
//    public void addAllInfos(final List<AliyunDownloadMediaInfo> downloadingMedias) {
//
//                for(AliyunDownloadMediaInfo downloadMediaInfo : downloadingMedias){
//                    new AsyncTask<AliyunDownloadMediaInfo , Void , String>(){
//                        AliyunDownloadMediaInfo info;
//
//                        @Override
//                        protected String doInBackground(AliyunDownloadMediaInfo... aliyunDownloadMediaInfos) {
//                            info = aliyunDownloadMediaInfos[0];
//                            return  PlayAuthUtil.getPlayAuth(mContext.getApplicationContext(), info.getVid());
//                        }
//
//                        @Override
//                        protected void onPostExecute(String playauth) {
//                            if(playauth == null){
//                                return;
//                            }
//
//                            AliyunPlayAuth.AliyunPlayAuthBuilder authBuilder = new AliyunPlayAuth.AliyunPlayAuthBuilder();
//                            authBuilder.setPlayAuth(playauth);
//                            authBuilder.setVid(info.getVid());
//                            authBuilder.setTitle(info.getTitle());
//                            authBuilder.setQuality(info.getQuality());
//                            authBuilder.setFormat(info.getFormat());
//                            authBuilder.setIsEncripted(info.isEncripted());
//
//                            addNewInfo(info, authBuilder.build());
//                        }
//                    }.execute(downloadMediaInfo);
//
//                }
//    }


    private boolean hasAdded(AliyunDownloadMediaInfo info) {
        for (AliyunDownloadMediaInfo downloadMediaInfo : downloadItems) {
            if (info.getFormat().equals(downloadMediaInfo.getFormat()) &&
                    info.getQuality().equals(downloadMediaInfo.getQuality()) &&
                    info.getVid().equals(downloadMediaInfo.getVid()) &&
                    info.isEncripted() == downloadMediaInfo.isEncripted()) {
                return true;
            }
        }
        return false;
    }

    private AliyunPlayAuth getPlayAuth(String format, String quality, String vid, int encripted) {
        for (AliyunDownloadMediaInfo downloadMediaInfo : downloadSourceHashMap.keySet()) {
            if (format.equals(downloadMediaInfo.getFormat()) &&
                    quality.equals(downloadMediaInfo.getQuality()) &&
                    vid.equals(downloadMediaInfo.getVid()) &&
                    encripted == downloadMediaInfo.isEncripted()) {
                return downloadSourceHashMap.get(downloadMediaInfo);
            }
        }

        return null;
    }


    private List<AliyunDownloadMediaInfo> downloadItems = new ArrayList<AliyunDownloadMediaInfo>();
    private HashMap<AliyunDownloadMediaInfo, AliyunPlayAuth> downloadSourceHashMap = new HashMap<AliyunDownloadMediaInfo, AliyunPlayAuth>();

    @Override
    public int getCount() {
        return downloadItems.size();
    }

    @Override
    public Object getItem(int position) {
        return downloadItems.get(position);
    }

    @Override
    public long getItemId(int position) {
        return position;
    }

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        DownloadItemHolder holder = null;
        if (convertView == null) {
            convertView = LayoutInflater.from(mContext.getApplicationContext()).inflate(R.layout.adapter_item_download, null);
            holder = new DownloadItemHolder(convertView);
            convertView.setTag(holder);
        } else {
            holder = (DownloadItemHolder) convertView.getTag();
        }

        AliyunDownloadMediaInfo info = downloadItems.get(position);
        holder.bindData(info);

        return convertView;
    }


    private AliyunRefreshPlayAuthCallback authCallback = new AliyunRefreshPlayAuthCallback() {
        //                        @RequiresApi(api = Build.VERSION_CODES.KITKAT)
        @Override
        public AliyunPlayAuth refreshPlayAuth(String vid, String quality, String format, String title, boolean encript) {

            return getPlayAuth(format, quality, vid, encript ? 1 : 0);

            //TODO 以上方法会有auth过期的问题。
            //TODO 以下是最好的解决方案：实时从服务器获取。
//            String playauth = PlayAuthUtil.getPlayAuth(mContext.getApplicationContext(), vid);
//            if (playauth == null) {
//                return null;
//            }
//            AliyunPlayAuth.AliyunPlayAuthBuilder authBuilder = new AliyunPlayAuth.AliyunPlayAuthBuilder();
//            authBuilder.setPlayAuth(playauth);
//            authBuilder.setVid(vid);
//            authBuilder.setTitle(title);
//            authBuilder.setQuality(quality);
//            authBuilder.setFormat(format);
//            authBuilder.setIsEncripted(encript ? 1 : 0);
//            return authBuilder.build();
        }
    };

    private class DownloadItemHolder {
        private ImageView cover;
        private TextView title, status;
        private ProgressBar progressBar;
        private Button start, stop, remove, play;

        public DownloadItemHolder(View convertView) {
            cover = (ImageView) convertView.findViewById(R.id.cover);
            title = (TextView) convertView.findViewById(R.id.title);
            status = (TextView) convertView.findViewById(R.id.status);
            progressBar = (ProgressBar) convertView.findViewById(R.id.progressbar);
            start = (Button) convertView.findViewById(R.id.start);
            stop = (Button) convertView.findViewById(R.id.stop);
            remove = (Button) convertView.findViewById(R.id.remove);
            play = (Button) convertView.findViewById(R.id.play);
        }


        public void bindData(final AliyunDownloadMediaInfo info) {
            Glide.with(mContext).load(info.getCoverUrl()).into(cover);
            if (info.isEncripted() == 1) {
                title.setText(info.getTitle() + "(" + info.getFormat() + "," + info.getQuality() + "," + info.getSizeStr() + ",加密" + ")");
            } else {
                title.setText(info.getTitle() + "(" + info.getFormat() + "," + info.getQuality() + "," + info.getSizeStr() + ")");
            }
            VcPlayerLog.d(TAG, "progress = " + info.getProgress());
            progressBar.setProgress(info.getProgress());
            status.setText(info.getStatus().toString());
            start.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    int success = downloadManager.startDownloadMedia(downloadSourceHashMap.get(info), authCallback);
                    if (success == AliyunErrorCode.ALIVC_ERR_DONWLOAD_NO_SPACE.getCode()) {
                        Toast.makeText(mContext.getApplicationContext(), "启动下载失败，下载列表中视频磁盘空间已满", Toast.LENGTH_LONG).show();
                    }
                }
            });
            stop.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    downloadManager.stopDownloadMedia(info);
                }
            });
            remove.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    downloadManager.removeDownloadMedia(info);
                    downloadItems.remove(info);
                    downloadSourceHashMap.remove(info);
                    notifyDataSetChanged();
                }
            });
            play.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    Intent intent = new Intent();
                    if (!(mContext instanceof Activity)) {
                        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                    }
                    intent.setClass(mContext, SkinActivity.class);
                    intent.putExtra("type", "localSource");
                    intent.putExtra("url", info.getSavePath());
                    mContext.startActivity(intent);
                }
            });

        }
    }

    public void updateInfo(AliyunDownloadMediaInfo aliyunDownloadMediaInfo) {
        AliyunDownloadMediaInfo tmpInfo = null;
        for (AliyunDownloadMediaInfo info : downloadItems) {
            if (info.getVid().equals(aliyunDownloadMediaInfo.getVid()) &&
                    info.getQuality().equals(aliyunDownloadMediaInfo.getQuality()) &&
                    info.getFormat().equals(aliyunDownloadMediaInfo.getFormat())) {
                tmpInfo = info;
                break;
            }
        }

        if (tmpInfo != null) {
            tmpInfo.setSavePath(aliyunDownloadMediaInfo.getSavePath());
            tmpInfo.setProgress(aliyunDownloadMediaInfo.getProgress());
            tmpInfo.setStatus(aliyunDownloadMediaInfo.getStatus());
        }
    }
}
